---
-- WorkModeMovingToolReset
--
-- Author: Wopster
-- Purpose: Resets the linked moving tool for the workMode.
--
-- Copyright (c) Wopster, 2019

---@class WorkModeMovingToolReset
WorkModeMovingToolReset = {}

function WorkModeMovingToolReset.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(WorkMode, specializations)
        and SpecializationUtil.hasSpecialization(Cylindered, specializations)
end

function WorkModeMovingToolReset.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "setWorkMode", WorkModeMovingToolReset.setWorkMode)
end

function WorkModeMovingToolReset.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", WorkModeMovingToolReset)
end

---Called on post load.
function WorkModeMovingToolReset:onPostLoad(savegame)
    local spec = self.spec_workMode
    for i, workMode in ipairs(spec.workModes) do
        local key = string.format("vehicle.workModes.workMode(%d)", i - 1)
        if not hasXMLProperty(self.xmlFile, key) then
            break
        end

        workMode.movingTools = {}
        local j = 0
        while true do
            local movingToolKey = string.format("%s.movingTools.movingTool(%d)", key, j)
            if not hasXMLProperty(self.xmlFile, movingToolKey) then
                break
            end

            local movingTool = {}
            movingTool.animationStopTime = getXMLFloat(self.xmlFile, movingToolKey .. "#animationStopTime")
            movingTool.node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, movingToolKey .. "#node"), self.i3dMappings)

            table.insert(workMode.movingTools, movingTool)

            j = j + 1
        end
    end
end

---Update the moving tool when the new work mode is set.
function WorkModeMovingToolReset:setWorkMode(superFunc, state, noEventSend)
    local spec = self.spec_workMode
    local oldState = spec.state
    superFunc(self, state, noEventSend)

    if self.isServer then
        if state ~= oldState then
            local newMode = spec.workModes[state]
            if newMode.movingTools == nil then
                return
            end

            for _, movingTool in ipairs(newMode.movingTools) do
                local tool = self.spec_cylindered.nodesToMovingTools[movingTool.node]
                if tool ~= nil then
                    if movingTool.animationStopTime ~= nil then
                        tool.curAnimTime = movingTool.animationStopTime
                        self:raiseDirtyFlags(tool.dirtyFlag)
                        self:raiseDirtyFlags(self.spec_cylindered.cylinderedDirtyFlag)
                    end
                end
            end
        end
    end
end
